# Utility class to run a Ycon instance and convert the strings returned
# into array structures suitable for further processing.
# It is particularly tailored to find URLs in either dragged links or
# bookmark files ('each_url'), but the 'each' method will process any
# dropped object (with the caveat below).

class BeDrop
 def initialize
  @p = IO.popen("Ycon")
 end

 # 'each_str' assembles the lines received from Ycon for each single
 # dropped item into one string and passes it to the associated block.
 # If there is irregular data in an item, it will get passed without
 # problem, but 'each' may not be able to handle it (below).
 def each_str
   mstr = 1
   until mstr == ""
    mstr = ""
    @p.each{|s| mstr<<s
     break if s == "]\n"
    }
    yield mstr
   end
 end
  
 # The 'each' method assembles and evaluates the lines received from Ycon
 # to create an Array structure representing the dropped message.
 # Caution: it expects "sensible" text strings -- which will normally be
 # the case for dropped-message entries.  However, a pasted clipboard item
 # could contain anything, and inappropriate characters such as extra quotes
 # may cause an error in 'eval'; if this happens it will print an error
 # and yield nil.
 def each
    each_str {|mstr|
    	begin
    		yield eval(mstr)
    	rescue SyntaxError
    		STDERR.puts "Unparseable string in message"
    		yield nil
    	end
    }
 end

 # 'each_url' looks for relevant fields in dropped links or bookmarks,
 # and extracts the URL string.  Yields nothing if no URL is found.
 def each_url
   each{|m| next if not m
     if ((u = m.assoc("be:url")) ||
      ((u = m.assoc("text/plain")) && u[1] == "MIME")) then yield u[2]
     elsif (u = m.assoc("refs")) && (u[1] == "RREF") then
      file = u[2][1]
      if %x(catattr BEOS:TYPE "#{file}").include? "application/x-vnd.Be-bookmark" then
      	yield %x(catattr META:url "#{file}").split(" : ")[-1].chop
       end
     end
   }
 end
end

